package models

type Employee struct {
	ID           int64   `json:"id" description:"员工id"`
	ImageURL     string  `json:"image_url" description:"员工照片"`
	EmployeeName string  `json:"employee_name" description:"员工名称"`
	RoleName     string  `json:"role_name" description:"职位"`
	JobState     int     `json:"job_state" description:"在职状态 0 在职 1离职"`
	Royalty      float64 `json:"royalty" description:"在职状态 0 在职 1离职"`
}

type EmployeeManage struct {
	SumMoney       float64    `json:"sum_money" description:"某段时间内所有业绩额"`
	SumRoyalty     float64    `json:"sum_royalty" description:"某段时间内所有员工的提成"`
	Employees      []Employee `json:"employees"  description:"员工列表及明细"`
}

type AnEmployee struct {
	SumMoney       float64          `json:"sum_money" description:"业绩额"`
	SumRoyalty     float64          `json:"sum_royalty" description:"提成"`
	WorkTrends     []*WorkTrendData `json:"work_trends" description:"工作趋势"`
}

type WorkTrend struct {
	Money       float64 `json:"money" description:"业绩额"`
	Royalty     float64 `json:"royalty" description:"提成"`
}

type WorkTrendData struct {
	Year  string  `json:"year"`
	Type  string  `json:"type"`
	Value float64 `json:"value"`
}

// 员工业绩 图表model
type EmployeeAchievement struct {
	Item   string  `json:"item" description:"商品一级分类"`
	//Bill   float64     `json:"bill" description:"商品数量"`
	Amount float64 `json:"amount" description:"金额"`
	Per    float64 `json:"per" description:"百分比"`
}

// 预约、已接诊、疫苗驱虫、美容洗澡、住院寄养
type EmployeeBusinessBill struct {
	Appointment     int `json:"appointment" description:"预约"`
	AlreadyAccepted int `json:"already_accepted" description:"已接诊"`
	VaccineInsect   int `json:"vaccine_insect" description:"疫苗驱虫"`
	Beauty          int `json:"beauty" description:"美容单"`
	HospitalFoster  int `json:"hospital_foster" description:"住院寄养"`
}

// 首页 数据
type TodayData struct {
	Turnover    float64 `json:"turnover" description:"今日营业额"`
	GrossProfit float64 `json:"gross_profit" description:"今日毛利"`
	Expenditure float64 `json:"expenditure" description:"今日支出"`
}

// 预约
type Appointment struct {
	AppointmentTime string `json:"appointment_time"`
	AppointmentName string `json:"appointment_name"`
	ConEmployeeName string `json:"con_employee_name"`
	HisPetName      string `json:"his_pet_name"`
	Species         string `json:"species"`
	Kind            string `json:"kind"`
	HisConsumerName string `json:"his_consumer_name"`
	Telephone       string `json:"telephone"`
}

// 已接诊
type Clinic struct {
	Time         string `json:"time"`
	EmployeeName string `json:"employee_name"`
	PetName      string `json:"pet_name"`
	Diagnosis    string `json:"diagnosis"`
}

// 疫苗驱虫
type ProtectionInsect struct {
	Time               string `json:"time"`
	EmployeeName       string `json:"employee_name"`
	PetName            string `json:"pet_name"`
	ProtectionOrInsect string `json:"protection_or_insect"`
}

// 美容
type Beauty struct {
	Time         string `json:"time"`
	EmployeeName string `json:"employee_name"`
	PetName      string `json:"pet_name"`
}

// 住院寄养
type HospitalFoster struct {
	HospitalDay      string `json:"hospital_day"`
	EmployeeName     string `json:"employee_name"`
	PetName          string `json:"pet_name"`
	DepositMoney     string `json:"deposit_money"`
	HospitalOrFoster string `json:"hospital_or_foster"`
}
