package models

// MyHospitals 医院列表model
type MyHospitals struct {
	ID      int64  `json:"id" description:"医院id"`
	Name    string `json:"name" description:"医院 名称"`
	LocalID int64  `json:"local_id" description:"本地ID"`
}

// UserInfo 用户信息
type UserInfo struct {
	ID           int64   `json:"id" description:"员工id"`
	ImageURL     string  `json:"image_url" description:"员工照片"`
	HospitalCode string  `json:"hospital_code" description:"医院code"`
	HospitalName string  `json:"hospital_name" description:"医院名称"`
	EmployeeName string  `json:"employee_name" description:"员工名称"`
	EmployeeSex  int64   `json:"employee_sex" description:"性别，0：男 1：女"`
	RoleName     string  `json:"role_name" description:"职位"`
	IsAdmin      uint    `json:"is_admin" description:"0 不是管理员, 1 管理员"`
	Help         float64 `json:"help" description:"帮助过的宠物"`
	Effect       string  `json:"effect" description:"影响力"`
	IsBindingWx  uint    `json:"is_binding_wx" description:"是否绑定微信"`
	IsDevice     uint    `json:"is_device" description:"是否是设备投放用户、0 不是、 1 是设备投放用户"`
}

// RolePermissionList 职位权限列表
type RolePermissionList struct {
	RoleID          int64  `json:"role_id" description:"职位ID, 0代表是管理员"`
	RoleName        string `json:"role_name" description:"职位"`
	PermissionCount int    `json:"permission_count" description:"权限数量, -1代表全部"`
}

// UserList 用户列表
type UserList struct {
	ID           int64  `json:"id" description:"员工id"`
	ImageURL     string `json:"image_url" description:"员工照片"`
	EmployeeName string `json:"employee_name" description:"员工名称"`
	RoleName     string `json:"role_name" description:"职位"`
	JobState     int    `json:"job_state" description:"在职状态 0 在职 1离职"`
}

// PermissionDetail 权限详情
type PermissionDetail struct {
	Permissions []WxDoctorModule `json:"permissions" description:"权限详情"`
	Users       []UserList       `json:"users" description:"用户列表"`
}

type LisOcrResponse struct {
	Code    int        `json:"code"`
	Message string     `json:"message"`
	Data    LisExamine `json:"data"`
}

type LisExamine struct {
	ExamineDetail []ExamineDetail `json:"examine_detail"`
	Time          string          `json:"time"`
	ImageUrl      string          `json:"image_url"`
	ExaminePid    string          `json:"examine_pid,omitempty"`    // 检验用户编号
	ExaminePname  string          `json:"examine_pname,omitempty"`  // 检验用户名（非必传输项）
	ExaminePtype  string          `json:"examine_ptype,omitempty"`  // 检验动物类型（非必传输项）
	DeviceType    string          `json:"device_type,omitempty"`    // 设备厂家
	DeviceFactory string          `json:"device_factory,omitempty"` // 设备类型
	DeviceName    string          `json:"device_name,omitempty"`    // 设备名称
	DeviceId      int             `json:"device_id,omitempty"`      // 设备id
	LotNumber     string          `json:"lot_number"`               // xx
	Serial        string          `json:"serial"`                   // xx
	Version       string          `json:"version"`                  // 版本
}
type ExamineDetail struct {
	Name  string `json:"name"`
	Unit  string `json:"unit"`
	Value string `json:"value"`
}

type LisImage struct {
	Url string `json:"url"`
}
